﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.Generic;

using Tao.OpenGl;
using Tao.Platform.Windows;

//using GLSharp;

namespace MultiSampling
{
    public partial class Form1 : Form
    {

        /// <summary>
        /// このフォームのデバイスコンテキストのハンドル
        /// </summary>
        IntPtr hDC;

        /// <summary>
        /// OpenGLレンダリングコンテキストのハンドル
        /// </summary>
        IntPtr hRC;

        /// <summary>
        /// レンダリングコンテキストのピクセルフォーマット
        /// </summary>
        Gdi.PIXELFORMATDESCRIPTOR pfd;

        /// <summary>
        /// ダブルバッファかどうかのフラグ
        /// </summary>
        bool doubleBuffer;


        public Form1()
        {
            InitializeComponent();
        }


        protected override void OnHandleCreated( EventArgs e )
        {
            base.OnHandleCreated( e );

            this.SetStyle( ControlStyles.UserPaint, true );
            this.SetStyle( ControlStyles.AllPaintingInWmPaint, true );
            this.SetStyle( ControlStyles.ResizeRedraw, true );


            //ダミーのコントロール
            Control dummyControl = new Control();

            //デバイスコンテキストのハンドルを取得。
            IntPtr dummyHDC = User.GetDC( dummyControl.Handle );

            //ピクセルフォーマットはとりあえず適当に設定。
            Gdi.PIXELFORMATDESCRIPTOR dummyPfd = new Gdi.PIXELFORMATDESCRIPTOR();
            dummyPfd.dwFlags = Gdi.PFD_SUPPORT_OPENGL | Gdi.PFD_DRAW_TO_WINDOW;
            dummyPfd.iPixelType = Gdi.PFD_TYPE_RGBA;
            dummyPfd.cColorBits = 24;
            dummyPfd.cColorBits = 0;

            //ピクセルフォーマットを選択
            int dummyPixFormat = Gdi.ChoosePixelFormat( dummyHDC, ref dummyPfd );

            if( dummyPixFormat <= 0 )
            {
                dummyControl.Dispose();
                throw new Exception( "ChoosePixelFormat failed." );
            }

            //ピクセルフォーマットの詳細を取得
            Wgl.wglDescribePixelFormat( dummyHDC, dummyPixFormat, 40, ref dummyPfd );

            //よくわからんが、これがないとエラーになる。
            Wgl.wglSwapBuffers( dummyHDC );

            //ピクセルフォーマットを設定。
            Gdi.SetPixelFormat( dummyHDC, dummyPixFormat, ref dummyPfd );

            //レンダリングコンテキストを作成。
            IntPtr dummyHRC = Wgl.wglCreateContext( dummyHDC );

            if( dummyHRC == IntPtr.Zero )
            {
                dummyControl.Dispose();
                throw new Exception( "Creating dummy context failed." );
            }

            //カレントに設定。
            Wgl.wglMakeCurrent( dummyHDC, dummyHRC );

            //WGL関数（エクステンション含む）のエントリポイントを取得する。
            //Tao.OpenGlを使う場合、これが必要になります。
            Wgl.ReloadFunctions();

            // WGLのエクステンション"WGL_EXT_extensions_string"がサポートされているかどうかを取得。
            bool supportWglExtString = Wgl.IsExtensionSupported( "WGL_EXT_extensions_string" );
            IntPtr ptr = Wgl.wglGetProcAddress( "wglGetExtensionsStringEXT" );

            if( !supportWglExtString )
            {
                MessageBox.Show( "WGL_EXT_extensions_string is not supported." );

                Wgl.wglDeleteContext( dummyHRC );
                dummyControl.Dispose();

                this.NormalyInitialize();
                return;
            }

            //WGLのエクステンションのリストを取得。
            List<string> wglExtensions = new List<string>( Wgl.wglGetExtensionsStringEXT().Split( new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries ) );


            bool supportWglArbPixFormat = wglExtensions.Contains( "WGL_ARB_pixel_format" );
            //bool supportWglArbPixFormat = Wgl.IsExtensionSupported( "WGL_ARB_pixel_format" );
            if( !supportWglArbPixFormat )
            {
                MessageBox.Show( "WGL_ARB_pixel_format is not supported." );

                Wgl.wglDeleteContext( dummyHRC );
                dummyControl.Dispose();

                this.NormalyInitialize();
                return;
            }

            bool supportWglArbMultisamples = wglExtensions.Contains( "WGL_ARB_multisample" );
            //bool supportWglArbMultisamples = Wgl.IsExtensionSupported( "WGL_ARB_multisample" );
            if( !supportWglArbMultisamples )
            {
                MessageBox.Show( "WGL_ARB_multisample is not supported." );

                Wgl.wglDeleteContext( dummyHRC );
                dummyControl.Dispose();

                this.NormalyInitialize();
                return;
            }

            //ピクセルフォーマットに要求する設定のリスト。
            // { 名前, 値, 名前, 値, ... 0, 0 } という順で並べる。
            int[] iAttribs = new int[]{ Wgl.WGL_SUPPORT_OPENGL_EXT, 1,  //0, 1,
                                        Wgl.WGL_DRAW_TO_WINDOW_EXT, 1,  //2, 3,
                                        Wgl.WGL_ACCELERATION_EXT, Wgl.WGL_FULL_ACCELERATION_EXT, //4, 5,
                                        Wgl.WGL_PIXEL_TYPE_EXT, Wgl.WGL_TYPE_RGBA_EXT, //6, 7,
                                        Wgl.WGL_COLOR_BITS_EXT, 24,     //8, 9,
                                        Wgl.WGL_ALPHA_BITS_EXT, 8,      //10, 11,
                                        Wgl.WGL_DEPTH_BITS_EXT, 16,     //12, 13,
                                        Wgl.WGL_DOUBLE_BUFFER_EXT, 1,   //14, 15,
                                        Wgl.WGL_SAMPLE_BUFFERS_EXT, 1,  //16, 17, // マルチサンプリングで使用するバッファを作成するかどうかの指定
                                        Wgl.WGL_SAMPLES_EXT, 4,         //18, 19, // マルチサンプリングのサンプル数。2, 4, 8, ... を指定可。
                                        0,0 };                          //20, 21  // 最後は 0, 0 としておかなければならないのは、仕様。

            // これに、ピクセルフォーマットのインデックスが返される。
            int[] formats = new int[1];

            // マッチしたフォーマントの数がこれに返される。
            int[] numFormats = new int[1];

            bool valid = Wgl.wglChoosePixelFormatARB( dummyHDC, iAttribs, null, formats.Length, formats, numFormats );

            if( !valid || ( numFormats[0] <= 0 ) )
            {
                //サンプリング数を2に減らして、リトライ。
                iAttribs[19] = 2;
                valid = Wgl.wglChoosePixelFormatARB( dummyHDC, iAttribs, null, formats.Length, formats, numFormats );
            }

            //これ以降、ダミーは不要となるので、
            //解放しておく。
            Wgl.wglDeleteContext( dummyHRC );
            dummyControl.Dispose();

            if( !valid || ( numFormats[0] <= 0 ) )
            {
                MessageBox.Show( "ChoosePixelFormatARB failed." );

                this.NormalyInitialize();
                return;
            }

            //本命のほうのデバイスコンテキストのハンドル。
            this.hDC = User.GetDC( this.Handle );
            //と、ピクセルフォーマット。
            this.pfd = new Gdi.PIXELFORMATDESCRIPTOR();

            //ピクセルフォーマットの設定を取得。
            Wgl.wglDescribePixelFormat( this.hDC, formats[0], 40, ref this.pfd );

            Wgl.wglSwapBuffers( this.hDC );

            //デバイスコンテキストにピクセルフォーマットを設定。
            valid = Gdi.SetPixelFormat( this.hDC, formats[0], ref this.pfd );

            if( !valid )
            {
                throw new Exception( "SetPixelFormat failed." );
            }

            //レンダリングコンテキストを作成。
            this.hRC = Wgl.wglCreateContext( this.hDC );
            if( this.hRC == IntPtr.Zero ) throw new Exception( "CreateContext failed." );

            //カレントに設定。
            Wgl.wglMakeCurrent( this.hDC, this.hRC );

            //Tao.OpenGl.Gl, Tao.Platform.Windows.Wglを使うときのお約束。
            Gl.ReloadFunctions();
            Wgl.ReloadFunctions();

            //マルチサンプリングを有効にする。
            Gl.glEnable( Gl.GL_MULTISAMPLE );

            SetupGL();

        }


        /// <summary>
        /// マルチサンプリング無しで、普通に初期化。
        /// </summary>
        private void NormalyInitialize()
        {
            this.pfd = new Gdi.PIXELFORMATDESCRIPTOR();
            this.pfd.dwFlags = Gdi.PFD_SUPPORT_OPENGL | Gdi.PFD_DRAW_TO_WINDOW | Gdi.PFD_DOUBLEBUFFER;
            this.pfd.iPixelType = Gdi.PFD_TYPE_RGBA;
            this.pfd.cColorBits = 32;
            this.pfd.cAlphaBits = 8;

            this.hDC = User.GetDC( this.Handle );

            int pixFormat = Gdi.ChoosePixelFormat( this.hDC, ref this.pfd );
            if( pixFormat <= 0 ) throw new Exception( "ChoosePixelFormat failed." );

            Wgl.wglDescribePixelFormat( this.hDC, pixFormat, 40, ref this.pfd );

            Wgl.wglSwapBuffers( this.hDC );

            Gdi.SetPixelFormat( this.hDC, pixFormat, ref this.pfd );
            this.hRC = Wgl.wglCreateContext( this.hDC );

            if( this.hRC == IntPtr.Zero ) throw new Exception( "Creating dummy context failed." );

            Wgl.wglMakeCurrent( this.hDC, this.hRC );

            Gl.ReloadFunctions();
            Wgl.ReloadFunctions();

            int err = Gl.glGetError();

            if( err != Gl.GL_NO_ERROR )
            {
                throw new Exception( "Error code = " + err.ToString() );
            }

            SetupGL();

        }


        private void SetupGL()
        {
            Gl.glClearColor( 0f, 0f, 0f, 1f );

            if( this.pfd.cAlphaBits > 0 )
            {
                Gl.glEnable( Gl.GL_BLEND );
                Gl.glBlendFunc( Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA );
            }

            Gl.glShadeModel( Gl.GL_SMOOTH );

            this.doubleBuffer = ( ( this.pfd.dwFlags & Gdi.PFD_DOUBLEBUFFER ) == Gdi.PFD_DOUBLEBUFFER );
        }


        /// <summary>
        /// レンダリングコンテキストを破棄します。
        /// </summary>
        /// <remarks>
        /// デザイナが生成したDisposeメソッドの中で呼び出されます。
        /// </remarks>
        private void DestroyContext()
        {
            Wgl.wglDeleteContext( this.hRC );
        }


        protected override void OnPaint( PaintEventArgs e )
        {
            //base.OnPaint( e );

            if( Wgl.wglGetCurrentContext() != this.hRC )
            {
                Wgl.wglMakeCurrent( this.hDC, this.hRC );
            }

            Gl.glClear( Gl.GL_COLOR_BUFFER_BIT );

            Gl.glMatrixMode( Gl.GL_PROJECTION );
            Gl.glLoadIdentity();
            Gl.glViewport( 0, 0, this.ClientRectangle.Width, this.ClientRectangle.Height );

            Gl.glMatrixMode( Gl.GL_MODELVIEW );
            Gl.glLoadIdentity();
            Gl.glRotatef( 20f, 0f, 0f, 1f );

            Gl.glBegin( Gl.GL_TRIANGLES );

            Gl.glColor3f( 1f, 0f, 0f );
            Gl.glVertex2f( -0.8f, -0.6f );

            Gl.glColor3f( 0f, 1f, 0f );
            Gl.glVertex2f( 0.8f, -0.6f );

            Gl.glColor3f( 0f, 0f, 1f );
            Gl.glVertex2f( 0.0f, 0.8f );


            Gl.glColor4f( 1f, 0f, 0f, 0.6f );
            Gl.glVertex2f( -0.8f, 0.6f );

            Gl.glColor4f( 0f, 1f, 0f, 0.6f );
            Gl.glVertex2f( 0.0f, -0.8f );

            Gl.glColor4f( 0f, 0f, 1f, 0.6f );
            Gl.glVertex2f( 0.8f, 0.6f );

            Gl.glEnd();

            if( this.doubleBuffer )
            {
                Wgl.wglSwapBuffers( this.hDC );
            }
            else
            {
                Gl.glFlush();
            }
        }


        protected override void OnPaintBackground( PaintEventArgs e )
        {
            //base.OnPaintBackground( e );
        }


    }


}
