﻿// 作成 : 2008 一歩二歩散歩
// WGLのエクステンション

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Drawing;
using System.Collections.Generic;

namespace GLSharp
{
    /// <summary>
    /// Accepted in the &lt;piAttributes&gt; parameter array of
    /// wglGetPixelFormatAttribivARB, and wglGetPixelFormatAttribfvARB, and
    /// as a type in the &lt;piAttribILis&gt; and &lt;pfAttribFList&gt; parameter
    /// arrays of wglChoosePixelFormatARB.
    /// </summary>
    public enum PFArbAttribI : int
    {
        /// <summary>
        /// The number of pixel formats for the device context. The
        /// &lt;iLayerPlane&gt; and &lt;iPixelFormat&gt; parameters are ignored if this
        /// attribute is specified.
        /// </summary>
        NumberPixelFormats = 0x2000,
        /// <summary>
        /// True if the pixel format can be used with a window. The
        /// &lt;iLayerPlane&gt; parameter is ignored if this attribute is
        /// specified.
        /// </summary>
        DrawToWindow = 0x2001,
        /// <summary>
        /// True if the pixel format can be used with a memory bitmap. The
        /// &lt;iLayerPlane&gt; parameter is ignored if this attribute is
        /// specified.
        /// </summary>
        DrawToBitmap = 0x2002,
        /// <summary>
        /// Indicates whether the pixel format is supported by the driver.
        /// If this is set to WGL_NO_ACCELERATION_ARB then only the software
        /// renderer supports this pixel format; if this is set to
        /// WGL_GENERIC_ACCELERATION_ARB then the pixel format is supported
        /// by an MCD driver; if this is set to WGL_FULL_ACCELERATION_ARB
        /// then the pixel format is supported by an ICD driver.
        /// </summary>
        Acceleration = 0x2003,
        /// <summary>
        /// A logical palette is required to achieve the best results for
        /// this pixel format. The &lt;iLayerPlane&gt; parameter is ignored if
        /// this attribute is specified.
        /// </summary>
        NeedPalette = 0x2004,
        /// <summary>
        /// The hardware supports one hardware palette in 256-color mode
        /// only. The &lt;iLayerPlane&gt; parameter is ignored if this attribute
        /// is specified.
        /// </summary>
        NeedSystemPalette = 0x2005,
        /// <summary>
        /// True if the pixel format supports swapping layer planes
        /// independently of the main planes. If the pixel format does not
        /// support a back buffer then this is set to FALSE. The
        /// &lt;iLayerPlane&gt; parameter is ignored if this attribute is
        /// specified.
        /// </summary>
        SwapLayerBuffers = 0x2006,
        /// <summary>
        /// If the pixel format supports a back buffer, then this indicates
        /// how they are swapped. If this attribute is set to
        /// WGL_SWAP_EXCHANGE_ARB then swapping exchanges the front and back
        /// buffer contents; if it is set to WGL_SWAP_COPY_ARB then swapping
        /// copies the back buffer contents to the front buffer; if it is
        /// set to WGL_SWAP_UNDEFINED_ARB then the back buffer contents are
        /// copied to the front buffer but the back buffer contents are
        /// undefined after the operation. If the pixel format does not
        /// support a back buffer then this parameter is set to
        /// WGL_SWAP_UNDEFINED_ARB. The &lt;iLayerPlane&gt; parameter is ignored
        /// if this attribute is specified.
        /// </summary>
        SwapMethod = 0x2007,
        /// <summary>
        /// The number of underlay planes. The &lt;iLayerPlane&gt; parameter is
        /// ignored if this attribute is specified.
        /// </summary>
        NumberOverlays = 0x2008,
        /// <summary>
        /// The number of underlay planes. The &lt;iLayerPlane&gt; parameter is
        /// ignored if this attribute is specified.
        /// </summary>
        NumberUnderlays = 0x2009,
        /// <summary>
        /// True if transparency is supported.
        /// </summary>
        Transparent = 0x200A,
        /// <summary>
        /// Specifies the transparent red color value. Typically this value
        /// is the same for all layer planes. This value is undefined if
        /// transparency is not supported.
        /// </summary>
        TransparentRedValue = 0x2037,
        /// <summary>
        /// Specifies the transparent green value. Typically this value is
        /// the same for all layer planes. This value is undefined if
        /// transparency is not supported.
        /// </summary>
        TransparentGreenValue = 0x2038,
        /// <summary>
        /// Specifies the transparent blue color value. Typically this value
        /// is the same for all layer planes. This value is undefined if
        /// transparency is not supported.
        /// </summary>
        TransparentBlueValue = 0x2039,
        /// <summary>
        /// Specifies the transparent alpha value. This is reserved for
        /// future use.
        /// </summary>
        TransparentAlphaValue = 0x203A,
        /// <summary>
        /// Specifies the transparent color index value. Typically this
        /// value is the same for all layer planes. This value is undefined
        /// if transparency is not supported.
        /// </summary>
        TransparentIndexValue = 0x203B,
        /// <summary>
        /// True if the layer plane shares the depth buffer with the main
        /// planes. If &lt;iLayerPlane&gt; is zero, this is always true.
        /// </summary>
        ShareDepth = 0x200C,
        /// <summary>
        /// True if the layer plane shares the stencil buffer with the main
        /// planes. If &lt;iLayerPlane&gt; is zero, this is always true.
        /// </summary>
        ShareStencil = 0x200D,
        /// <summary>
        /// True if the layer plane shares the accumulation buffer with the
        /// main planes. If &lt;iLayerPlane&gt; is zero, this is always true.
        /// </summary>
        ShareAccum = 0x200E,
        /// <summary>
        /// True if GDI rendering is supported.
        /// </summary>
        SupportGdi = 0x200F,
        /// <summary>
        /// True if OpenGL is supported.
        /// </summary>
        SupportOpengl = 0x2010,
        /// <summary>
        /// True if the color buffer has back/front pairs.
        /// </summary>
        DoubleBuffer = 0x2011,
        /// <summary>
        /// True if the color buffer has left/right pairs.
        /// </summary>
        Stereo = 0x2012,
        /// <summary>
        /// The type of pixel data. This can be set to WGL_TYPE_RGBA_ARB or
        /// WGL_TYPE_COLORINDEX_ARB.
        /// </summary>
        PixelType = 0x2013,
        /// <summary>
        /// The number of color bitplanes in each color buffer. For RGBA
        /// pixel types, it is the size of the color buffer, excluding the
        /// alpha bitplanes. For color-index pixels, it is the size of the
        /// color index buffer.
        /// </summary>
        ColorBits = 0x2014,
        /// <summary>
        /// The number of red bitplanes in each RGBA color buffer.
        /// </summary>
        RedBits = 0x2015,
        /// <summary>
        /// The shift count for red bitplanes in each RGBA color buffer.
        /// </summary>
        RedShift = 0x2016,
        /// <summary>
        /// The number of green bitplanes in each RGBA color buffer.
        /// </summary>
        GreenBits = 0x2017,
        /// <summary>
        /// The shift count for green bitplanes in each RGBA color buffer.
        /// </summary>
        GreenShift = 0x2018,
        /// <summary>
        /// The number of blue bitplanes in each RGBA color buffer.
        /// </summary>
        BlueBits = 0x2019,
        /// <summary>
        /// The shift count for blue bitplanes in each RGBA color buffer.
        /// </summary>
        BlueShift = 0x201A,
        /// <summary>
        /// The number of alpha bitplanes in each RGBA color buffer.
        /// </summary>
        AlphaBits = 0x201B,
        /// <summary>
        /// The shift count for alpha bitplanes in each RGBA color buffer.
        /// </summary>
        AlphaShift = 0x201C,
        /// <summary>
        /// The total number of bitplanes in the accumulation buffer.
        /// </summary>
        AccumBits = 0x201D,
        /// <summary>
        /// The number of red bitplanes in the accumulation buffer.
        /// </summary>
        AccumRedBits = 0x201E,
        /// <summary>
        /// The number of green bitplanes in the accumulation buffer.
        /// </summary>
        AccumGreenBits = 0x201F,
        /// <summary>
        /// The number of blue bitplanes in the accumulation buffer.
        /// </summary>
        AccumBlueBits = 0x2020,
        /// <summary>
        /// The number of alpha bitplanes in the accumulation buffer.
        /// </summary>
        AccumAlphaBits = 0x2021,
        /// <summary>
        /// The depth of the depth (z-axis) buffer.
        /// </summary>
        DepthBits = 0x2022,
        /// <summary>
        /// The depth of the stencil buffer.
        /// </summary>
        StencilBits = 0x2023,
        /// <summary>
        /// The number of auxiliary buffers.
        /// </summary>
        AuxBuffers = 0x2024,

        /// <summary>
        /// The number of multisample buffers.
        /// </summary>
        SampleBuffers = 0x2041,

        /// <summary>
        /// The number of samples per pixel.
        /// </summary>
        Samples = 0x2042
    }

    /// <summary>
    /// Accepted as a value in the &lt;piAttribIList&gt; and &lt;pfAttribFList&gt;
    /// parameter arrays of wglChoosePixelFormatARB, and returned in the
    /// &lt;piValues&gt; parameter array of wglGetPixelFormatAttribivARB, and the
    /// &lt;pfValues&gt; parameter array of wglGetPixelFormatAttribfvARB
    /// </summary>
    public enum PFArbAttribValue : int
    {
        NoAcceleration = 0x2025,
        GenericAcceleration = 0x2026,
        FullAcceleration = 0x2027,
        SwapExchange = 0x2028,
        SwapCopy = 0x2029,
        SwapUndefined = 0x202A,
        TypeRgba = 0x202B,
        TypeColorindex = 0x202C
    }

    public enum PFAPixelType
    {
        TypeRgba = 0x202B,
        TypeColorindex = 0x202C
    }

    public enum PFASwapMethod
    {
        SwapExchange = 0x2028,
        SwapCopy = 0x2029,
        SwapUndefined = 0x202A
    }

    public enum PFAAccelerationType
    {
        NoAcceleration = 0x2025,
        GenericAcceleration = 0x2026,
        FullAcceleration = 0x2027
    }
}